// JavaTalkServer.java
//
// CS549 Spring 96, Oregon State University
// 
// Jean-Guy Speton, Maharajapuram Balakrishnan
//
// This file adapted from Server.java, from the book _Java in a Nutshell_
// by David Flanagan, published by O'Reilly & Associates.

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * @see    JavaTalkClient
 * @author Jean-Guy Speton
 * @author Maharajapuram Balakrishnan
 *
 * The JavaTalkServer class contains the ServerSocket which listens on
 * the DEFAULT_PORT for JavaTalk connections and, upon accepting a
 * connection, spawns a JavaTalkConnection to handle the connection
 * details.
 */
public class JavaTalkServer extends Thread
{
  private static final String VERSION_STRING =
    "JavaTalk 1.01 Copyright (C) 1996-7.  All rights reserved.";

  public final static int DEFAULT_PORT = 8421;

  private static boolean log = false;
  
  protected int port;
  protected ServerSocket listen_socket;
  protected ThreadGroup threadgroup;
  protected Vector connections;
    
  // Exit with an error message, when an exception occurs.
  public static void fail(Exception e, String msg)
  {
    System.err.println(msg + ": " +  e);
    System.exit(1);
  }
    
  // Create a ServerSocket to listen for connections on;  start the thread.
  public JavaTalkServer(int port)
  {
    // Create our server thread with a name.
    super("JavaTalkServer");

    this.port = port;

    try {
      listen_socket = new ServerSocket(port);
    }
    catch (IOException e) {
      fail(e, "Exception creating server socket");
    }

    // Create a threadgroup for our connections
    threadgroup = new ThreadGroup("JavaTalk Server Connections");

    // Initialize a vector to store our connections in
    connections = new Vector();

    // Start the server listening for connections
    this.start();
  }
    
  // The body of the server thread.  Loop forever, listening for and
  // accepting connections from clients.  For each connection, 
  // create a Connection object to handle communication through the
  // new Socket.  When we create a new connection, add it to the
  // Vector of connections.  Note that we
  // use synchronized to lock the Vector of connections.
  public void run()
  {
    System.out.println("JavaTalkServer successfully initialized.");
    System.out.println("Listening to port " + port + "...");

    try {
      while(true) {
	Socket client_socket = listen_socket.accept();
	JavaTalkConnection c = new JavaTalkConnection(client_socket,
						      threadgroup,
						      Thread.NORM_PRIORITY,
						      this);
	// prevent simultaneous access.
	synchronized (connections) {
	  connections.addElement(c);
	}

	// Since JavaTalkConnection.start() will broadcast the user
	// list to all connections, including itself, we must start
	// the thread after we have added it to the vector.
	c.start();
      }
    }
    catch (IOException e) {
      fail(e, "Exception while listening for connections");
    }
  }

  public void log(String s)
  {
    if (log) {
      System.out.println(s);
    }
  }
  
  // Start the server up, listening on an optionally specified port
  public static void main(String[] args)
  {
    System.out.println(VERSION_STRING);
    
    int port = DEFAULT_PORT;;

    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-h")) {
	System.out.println("Usage: java JavaTalkServer [-h] [-v] [port]");
	System.out.println("  -h    Display this help text.");
	System.out.println("  -v    Produce verbose output, mostly useful for debugging.");
	System.out.println("  port  Port to listen for connections on.  Defaults to " + DEFAULT_PORT + ".");
	System.exit(0);
      }
      else if (args[i].equals("-v")) {
	log = true;
      }
      else {
	try {
	  port = Integer.parseInt(args[i]);
	}
	catch (NumberFormatException e) {
	  port = 0;
	}
      }
    }
    
    new JavaTalkServer(port);
  }
}
